@@ -16,7 +16,6 @@ from os.path import splitext, basename
 import h5py
 import mrcfile
 import numpy as np
-import skimage.io as skio
 
 from scipy.ndimage import (
     zoom,
@@ -26,7 +25,6 @@ from scipy.ndimage import (
     binary_erosion,
     generic_gradient_magnitude,
 )
-from scipy.spatial import ConvexHull
 
 from .types import NDArray
 from .rotations import align_to_axis
@@ -36,6 +34,7 @@ from .matching_utils import (
     array_to_memmap,
     memmap_to_array,
     minimum_enclosing_box,
+    is_gzipped,
 )
 
 __all__ = ["Density"]
@@ -331,6 +330,7 @@ class Density:
         if non_standard_crs:
             data = np.transpose(data, crs_index)
             origin = np.take(origin, crs_index)
+            sampling_rate = np.take(sampling_rate, crs_index)
 
         return data.T, origin[::-1], sampling_rate[::-1], metadata
 
@@ -569,6 +569,8 @@ class Density:
         --------
         :py:meth:`Density.from_file`
         """
+        import skimage.io as skio
+
         swap = filename
         if is_gzipped(filename):
             with gzip_open(filename, "rb") as infile:
@@ -936,6 +938,8 @@ class Density:
         ----------
         .. [1] https://scikit-image.org/docs/stable/api/skimage.io.html
         """
+        import skimage.io as skio
+
         swap, kwargs = filename, {}
         if gzip:
             swap = BytesIO()
@@ -1401,8 +1405,7 @@ class Density:
         cutoff : float
             Above this value arr elements are considered. Defaults to 0.
         use_geometric_center : bool, optional
-            Whether the box should accommodate the geometric or the coordinate
-            center. Defaults to False.
+            Accommodate the geometric instead of the mass center.
 
         Returns
         -------
@@ -1414,24 +1417,25 @@ class Density:
         :py:meth:`Density.adjust_box`
         :py:meth:`tme.matching_utils.minimum_enclosing_box`
         """
-        coordinates = self.to_pointcloud(threshold=cutoff)
-        starts, stops = coordinates.min(axis=1), coordinates.max(axis=1)
+        if cutoff is None:
+            cutoff = self.data.min() - 1
 
+        coordinates = self.to_pointcloud(threshold=cutoff)
         shape = minimum_enclosing_box(
             coordinates=coordinates,
             use_geometric_center=use_geometric_center,
         )
-        difference = np.maximum(np.subtract(shape, np.subtract(stops, starts)), 0)
 
-        shift_start = np.divide(difference, 2).astype(int)
-        shift_stop = shift_start + np.mod(difference, 2)
+        starts, stops = coordinates.min(axis=1), coordinates.max(axis=1)
+        diff = np.maximum(np.subtract(shape, np.subtract(stops, starts)), 0)
 
+        shift_start = np.divide(diff, 2).astype(int)
+        shift_stop = shift_start + np.mod(diff, 2)
+
+        # These are purposefully negative to indicate left and right pad
         starts = (starts - shift_start).astype(int)
         stops = (stops + shift_stop).astype(int)
-
-        enclosing_box = tuple(slice(start, stop) for start, stop in zip(starts, stops))
-
-        return tuple(enclosing_box)
+        return tuple(slice(start, stop) for start, stop in zip(starts, stops))
 
     def pad(
         self, new_shape: Tuple[int], center: bool = True, padding_value: float = 0
@@ -1498,7 +1502,9 @@ class Density:
 
         self.adjust_box(new_box, pad_kwargs={"constant_values": padding_value})
 
-    def centered(self, cutoff: float = 0) -> Tuple["Density", NDArray]:
+    def centered(
+        self, cutoff: float = 0.0, order: int = 1
+    ) -> Tuple["Density", NDArray]:
         """
         Shifts the data center of mass to the center of the data array using linear
         interpolation. The box size of the returned :py:class:`Density` object is at
@@ -1507,8 +1513,10 @@ class Density:
         Parameters
         ----------
         cutoff : float, optional
-            Only elements in data larger than cutoff will be considered for
-            computing the new box. By default considers only positive elements.
+            Consider all elements larger than cutoff for computing the new box.
+            Default is 0.0.
+        order : int, optional
+            Interpolation order, defaults to 1.
 
         Notes
         -----
@@ -1532,16 +1540,15 @@ class Density:
         Examples
         --------
         :py:meth:`Density.centered` returns a tuple containing a centered version
-        of the current :py:class:`Density` instance, as well as an array with
-        translations. The translation corresponds to the shift between the original and
-        current center of mass.
+        of the current :py:class:`Density` instance. Centering is achieved via padding
+        and rigid-transform of the internal :py:attr:`Density.data` attribute.
+        `centered_dens` is sufficiently large to represent all rotations of the
+        :py:attr:`Density.data` attribute.
 
         >>> import numpy as np
         >>> from tme import Density
         >>> dens = Density(np.ones((5,5,5)))
-        >>> centered_dens, translation = dens.centered(0)
-        >>> translation
-        array([0., 0., 0.])
+        >>> centered_dens = dens.centered(0)
 
         :py:meth:`Density.centered` extended the :py:attr:`Density.data` attribute
         of the current :py:class:`Density` instance and modified
@@ -1549,22 +1556,6 @@ class Density:
 
         >>> centered_dens
         Origin: (-2.0, -2.0, -2.0), sampling_rate: (1, 1, 1), Shape: (9, 9, 9)
-
-        :py:meth:`Density.centered` achieves centering via zero-padding and
-        rigid-transform of the internal :py:attr:`Density.data` attribute.
-        `centered_dens` is sufficiently large to represent all rotations of the
-        :py:attr:`Density.data` attribute, such as ones obtained from
-        :py:meth:`tme.matching_utils.get_rotation_matrices`.
-
-        >>> from tme.matching_utils import get_rotation_matrices
-        >>> rotation_matrix = get_rotation_matrices(dim = 3 ,angular_sampling = 10)[0]
-        >>> rotated_centered_dens = centered_dens.rigid_transform(
-        >>>     rotation_matrix = rotation_matrix,
-        >>>     order = None
-        >>> )
-        >>> print(centered_dens.data.sum(), rotated_centered_dens.data.sum())
-        125.0 125.0
-
         """
         ret = self.copy()
 
@@ -1581,12 +1572,11 @@ class Density:
         ret = ret.rigid_transform(
             translation=shift,
             rotation_matrix=np.eye(ret.data.ndim),
-            use_geometric_center=False,
-            order=1,
+            use_geometric_center=True,
+            order=order,
         )
-
-        shift = np.subtract(center, self.center_of_mass(ret.data, cutoff))
-        return ret, shift
+        ret.origin = np.subtract(ret.origin, np.multiply(shift, ret.sampling_rate))
+        return ret
 
     def rigid_transform(
         self,
@@ -1893,6 +1883,8 @@ class Density:
 
         lower_bound, upper_bound = density_boundaries
         if method == "ConvexHull":
+            from scipy.spatial import ConvexHull
+
             binary = np.transpose(np.where(self.data > lower_bound))
             hull = ConvexHull(binary)
             surface_points = binary[hull.vertices[:]]
@@ -2030,27 +2022,24 @@ class Density:
             eroded_mask = binary_erosion(eroded_mask)
         return core_indices
 
-    @staticmethod
-    def center_of_mass(arr: NDArray, cutoff: float = None) -> NDArray:
+    def center_of_mass(self, arr: NDArray = None, cutoff: float = None) -> NDArray:
         """
-        Computes the center of mass of a numpy ndarray instance using all available
-        elements. For template matching it typically makes sense to only input
-        positive densities.
+        Computes the center of mass of a numpy ndarray instance.
 
         Parameters
         ----------
-        arr : NDArray
-            Array to compute the center of mass of.
+        arr : NDArray, optional
+            Array to compute the center of mass of, default is :py:attr:`Density.data`.
         cutoff : float, optional
-            Densities less than or equal to cutoff are nullified for center
-            of mass computation. By default considers all values.
+            Density cutoff for calculation. Defaults to None
 
         Returns
         -------
         NDArray
             Center of mass with shape (arr.ndim).
         """
-        return NumpyFFTWBackend().center_of_mass(arr**2, cutoff)
+        arr = self.data if arr is None else arr
+        return NumpyFFTWBackend().center_of_mass(arr, cutoff)
 
     @classmethod
     def match_densities(
@@ -2099,13 +2088,13 @@ class Density:
         -----
         No densities below cutoff_template are present in the returned Density object.
         """
-        from .matching_utils import normalize_template
+        from .matching_utils import standardize
         from .matching_optimization import optimize_match, create_score_object
 
         template_mask = template.empty
         template_mask.data.fill(1)
 
-        normalize_template(
+        template.data = standardize(
             template=template.data,
             mask=template_mask.data,
             n_observations=template_mask.data.sum(),
@@ -2139,8 +2128,8 @@ class Density:
         template_coordinates = template_coordinates * template_scaling[:, None]
 
         mass_center_difference = np.subtract(
-            cls.center_of_mass(target.data, cutoff_target),
-            cls.center_of_mass(template.data, cutoff_template),
+            target.center_of_mass(target.data, cutoff_target),
+            target.center_of_mass(template.data, cutoff_template),
         ).astype(int)
         template_coordinates += mass_center_difference[:, None]
 
@@ -2194,7 +2183,7 @@ class Density:
 
         Parameters
         ----------
-        target : Density
+        target : :py:class:`Density`
             The target map for template matching.
         template : Structure
             The template that should be aligned to the target.
@@ -2258,8 +2247,59 @@ class Density:
         weights = self.data[tuple(coordinates)]
         return align_to_axis(coordinates.T, weights=weights, axis=axis, flip=flip)
 
+    @staticmethod
+    def fourier_shell_correlation(density1: "Density", density2: "Density") -> NDArray:
+        """
+        Computes the Fourier Shell Correlation (FSC) between two instances of `Density`.
 
-def is_gzipped(filename: str) -> bool:
-    """Check if a file is a gzip file by reading its magic number."""
-    with open(filename, "rb") as f:
-        return f.read(2) == b"\x1f\x8b"
+        The Fourier transforms of the input maps are divided into shells
+        based on their spatial frequency. The correlation between corresponding shells
+        in the two maps is computed to give the FSC.
+
+        Parameters
+        ----------
+        density1 : :py:class:`Density`
+            Reference for comparison.
+        density2 : :py:class:`Density`
+            Target for comparison.
+
+        Returns
+        -------
+        NDArray
+            An array of shape (N, 2), where N is the number of shells.
+            The first column represents the spatial frequency for each shell
+            and the second column represents the corresponding FSC.
+
+        References
+        ----------
+        .. [1] https://github.com/tdgrant1/denss/blob/master/saxstats/saxstats.py
+        """
+        side = density1.data.shape[0]
+        df = 1.0 / side
+
+        qx_ = np.fft.fftfreq(side) * side * df
+        qx, qy, qz = np.meshgrid(qx_, qx_, qx_, indexing="ij")
+        qr = np.sqrt(qx**2 + qy**2 + qz**2)
+
+        qmax = np.max(qr)
+        qstep = np.min(qr[qr > 0])
+        nbins = int(qmax / qstep)
+        qbins = np.linspace(0, nbins * qstep, nbins + 1)
+        qbin_labels = np.searchsorted(qbins, qr, "right") - 1
+
+        F1 = np.fft.fftn(density1.data)
+        F2 = np.fft.fftn(density2.data)
+
+        qbin_labels = qbin_labels.reshape(-1)
+        numerator = np.bincount(
+            qbin_labels, weights=np.real(F1 * np.conj(F2)).reshape(-1)
+        )
+        term1 = np.bincount(qbin_labels, weights=np.abs(F1).reshape(-1) ** 2)
+        term2 = np.bincount(qbin_labels, weights=np.abs(F2).reshape(-1) ** 2)
+        np.multiply(term1, term2, out=term1)
+        denominator = np.sqrt(term1)
+        FSC = np.divide(numerator, denominator)
+
+        qidx = np.where(qbins < qx.max())
+
+        return np.vstack((qbins[qidx], FSC[qidx])).T
