@@ -520,33 +520,6 @@ class Preprocessor:
 
         return template
 
-    def mipmap_filter(self, template: NDArray, level: int) -> NDArray:
-        """
-        Perform mip map antialiasing filtering.
-
-        Parameters
-        ----------
-        template : NDArray
-            The input atomic structure map.
-        level : int
-            Pyramid layer. Resolution decreases cubically with level.
-
-        Returns
-        -------
-        NDArray
-            Simulated electron densities.
-        """
-        array = template.copy()
-        interpolation_box = array.shape
-
-        for k in range(template.ndim):
-            array = ndimage.decimate(array, q=level, axis=k)
-
-        template = ndimage.zoom(array, np.divide(template.shape, array.shape))
-        template = self.interpolate_box(box=interpolation_box, arr=template)
-
-        return template
-
     def interpolate_box(
         self, arr: NDArray, box: Tuple[int], kind: str = "nearest"
     ) -> NDArray:
@@ -588,218 +561,6 @@ class Preprocessor:
 
         return arr
 
-    def bandpass_filter(
-        self,
-        template: NDArray,
-        lowpass: float,
-        highpass: float,
-        sampling_rate: NDArray = None,
-        gaussian_sigma: float = 0.0,
-    ) -> NDArray:
-        """
-        Apply a band-pass filter on the provided template, using a
-        Butterworth approximation.
-
-        Parameters
-        ----------
-        template : NDArray
-            The input numpy array on which the band-pass filter should be applied.
-        lowpass : float
-            The lower boundary of the frequency range to be preserved. Lower values will
-            retain broader, more global features.
-        highpass : float
-            The upper boundary of the frequency range to be preserved.  Higher values
-            will emphasize finer details and potentially noise.
-        sampling_rate : NDarray, optional
-            The sampling rate along each dimension.
-        gaussian_sigma : float, optional
-            Sigma value for the gaussian smoothing to be applied to the filter.
-
-        Returns
-        -------
-        NDArray
-            Bandpass filtered numpy array.
-        """
-        bpf = self.bandpass_mask(
-            shape=template.shape,
-            lowpass=lowpass,
-            highpass=highpass,
-            sampling_rate=sampling_rate,
-            gaussian_sigma=gaussian_sigma,
-            omit_negative_frequencies=False,
-        )
-
-        fft_data = np.fft.fftn(template)
-        np.multiply(fft_data, bpf, out=fft_data)
-        ret = np.real(np.fft.ifftn(fft_data))
-        return ret
-
-    def bandpass_mask(
-        self,
-        shape: Tuple[int],
-        lowpass: float,
-        highpass: float,
-        sampling_rate: NDArray = None,
-        gaussian_sigma: float = 0.0,
-        omit_negative_frequencies: bool = True,
-    ) -> NDArray:
-        """
-        Compute an approximate Butterworth bundpass filter. The returned filter
-        has it's DC component at the origin.
-
-        Parameters
-        ----------
-        shape : tuple of ints
-            Shape of the returned bandpass filter.
-        lowpass : float
-            The lower boundary of the frequency range to be preserved. Lower values will
-            retain broader, more global features.
-        maximum_frequency : float
-            The upper boundary of the frequency range to be preserved.  Higher values
-            will emphasize finer details and potentially noise.
-        sampling_rate : NDarray, optional
-            The sampling rate along each dimension.
-        gaussian_sigma : float, optional
-            Sigma value for the gaussian smoothing to be applied to the filter.
-        omit_negative_frequencies : bool, optional
-            Whether the wedge mask should omit negative frequencies, i.e. be
-            applicable to non hermitian-symmetric fourier transforms.
-
-        Returns
-        -------
-        NDArray
-            Bandpass filtered.
-        """
-        from .filters import BandPassReconstructed
-
-        return BandPassReconstructed(
-            sampling_rate=sampling_rate,
-            lowpass=lowpass,
-            highpass=highpass,
-            return_real_fourier=omit_negative_frequencies,
-            use_gaussian=gaussian_sigma == 0.0,
-        )(shape=shape)["data"]
-
-    def step_wedge_mask(
-        self,
-        shape: Tuple[int],
-        tilt_angles: Tuple[float] = None,
-        opening_axis: int = 0,
-        tilt_axis: int = 2,
-        weights: float = None,
-        infinite_plane: bool = False,
-        omit_negative_frequencies: bool = True,
-    ) -> NDArray:
-        """
-        Create a wedge mask with the same shape as template by rotating a
-        plane according to tilt angles. The DC component of the filter is at the origin.
-
-        Parameters
-        ----------
-        tilt_angles : tuple of float
-            Sequence of tilt angles.
-        shape : Tuple of ints
-            Shape of the output wedge array.
-        tilt_axis : int, optional
-            Axis that the plane is tilted over.
-            - 0 for Z-axis
-            - 1 for Y-axis
-            - 2 for X-axis
-        opening_axis : int, optional
-            Axis running through the void defined by the wedge.
-            - 0 for Z-axis
-            - 1 for Y-axis
-            - 2 for X-axis
-        sigma : float, optional
-            Standard deviation for Gaussian kernel used for smoothing the wedge.
-        weights : float, tuple of float
-            Weight of each element in the wedge. Defaults to one.
-        omit_negative_frequencies : bool, optional
-            Whether the wedge mask should omit negative frequencies, i.e. be
-            applicable to symmetric Fourier transforms (see :obj:`numpy.fft.fftn`)
-
-        Returns
-        -------
-        NDArray
-            A numpy array containing the wedge mask.
-
-        See Also
-        --------
-        :py:meth:`Preprocessor.continuous_wedge_mask`
-        """
-        from .filters import WedgeReconstructed
-
-        return WedgeReconstructed(
-            angles=tilt_angles,
-            tilt_axis=tilt_axis,
-            opening_axis=opening_axis,
-            frequency_cutoff=None if infinite_plane else 0.5,
-            create_continuous_wedge=False,
-            weights=weights,
-            weight_wedge=weights is not None,
-        )(shape=shape, return_real_fourier=omit_negative_frequencies,)["data"]
-
-    def continuous_wedge_mask(
-        self,
-        start_tilt: float,
-        stop_tilt: float,
-        shape: Tuple[int],
-        opening_axis: int = 0,
-        tilt_axis: int = 2,
-        infinite_plane: bool = True,
-        omit_negative_frequencies: bool = True,
-    ) -> NDArray:
-        """
-        Generate a wedge in a given shape based on specified tilt angles and axis.
-        The DC component of the filter is at the origin.
-
-        Parameters
-        ----------
-        start_tilt : float
-            Starting tilt angle in degrees, e.g. a stage tilt of 70 degrees
-            would yield a start_tilt value of 70.
-        stop_tilt : float
-            Ending tilt angle in degrees, , e.g. a stage tilt of -70 degrees
-            would yield a stop_tilt value of 70.
-        tilt_axis : int
-            Axis that the plane is tilted over.
-            - 0 for Z-axis
-            - 1 for Y-axis
-            - 2 for X-axis
-        opening_axis : int
-            Axis running through the void defined by the wedge.
-            - 0 for Z-axis
-            - 1 for Y-axis
-            - 2 for X-axis
-        shape : Tuple of ints
-            Shape of the output wedge array.
-        omit_negative_frequencies : bool, optional
-            Whether the wedge mask should omit negative frequencies, i.e. be
-            applicable to symmetric Fourier transforms (see :obj:`numpy.fft.fftn`)
-        infinite_plane : bool, optional
-            Whether the plane should be considered to be larger than the shape. In this
-            case the output wedge mask fill have no spheric component.
-
-        Returns
-        -------
-        NDArray
-            Array of the specified shape with the wedge created based on
-            the tilt angles.
-
-        See Also
-        --------
-        :py:meth:`Preprocessor.step_wedge_mask`
-        """
-        from .filters import WedgeReconstructed
-
-        return WedgeReconstructed(
-            angles=(start_tilt, stop_tilt),
-            tilt_axis=tilt_axis,
-            opening_axis=opening_axis,
-            frequency_cutoff=None if infinite_plane else 0.5,
-            create_continuous_wedge=True,
-        )(shape=shape, return_real_fourier=omit_negative_frequencies)["data"]
-
 
 def window_kaiserb(width: int, beta: float = 3.2, order: int = 0) -> NDArray:
     """
